@@ -1,6 +1,7 @@
-import collections
 from collections import OrderedDict
+from collections.abc import Set
 from typing import Union, List, Any, Callable, Dict, Iterable, Tuple, Optional, Hashable
+from datetime import datetime
 
 
 def head(obj: Union[dict, set], top: int = 10) -> Union[dict, set]:
@@ -44,8 +45,8 @@ def list_union(*lists: list) -> list:
     :return: The union of all lists respecting the element order.
     """
     result = []
-    for l in lists:
-        for e in l:
+    for lst in lists:
+        for e in lst:
             if e not in result:
                 result.append(e)
     return result
@@ -300,7 +301,7 @@ def merge_tuples(tuples: Iterable[tuple]) -> tuple:
     return result
 
 
-class OrderedSet(collections.Set, Iterable):
+class OrderedSet(Set, Iterable):
     """ An ordered set with pop() method, which can be extracted using FIFO or LIFO. """
 
     def __init__(self, items: Iterable = iter([])) -> None:
@@ -325,7 +326,83 @@ class OrderedSet(collections.Set, Iterable):
         :param item: The element to add.
         """
         if item not in self.items:
-            self.items[item] = True
+            self.items[item] = datetime.now()
+
+    def time(self, item: Hashable) -> datetime:
+        """ Get the time when an element was added.
+
+        :param item: The item to search the time.
+        :return: A datetime object with the time where the object was introduced in the set.
+        """
+        return self.items[item]
+
+    def __getitem__(self, item: Hashable) -> datetime:
+        """ Get the time when an element was added.
+
+        :param item: The item to search the time.
+        :return: A datetime object with the time when the object was introduced in the set.
+        """
+        return self.time(item)
+
+    def set_time(self, item: Hashable, date: datetime) -> None:
+        """ Set the time of an element in the set.
+
+        :param item: The item to modify its introduction date.
+        :param date: The date to modify.
+        """
+        self.items[item] = date
+
+    def __setitem__(self, item: Hashable, date: datetime) -> None:
+        """ Set the time of an element in the set.
+
+        :param item: The item to modify its introduction date.
+        :param date: The date to modify.
+        """
+        self.set_time(item, date)
+
+    def before(self, date: datetime) -> 'OrderedSet':
+        """ Get a copy of the OrderedSet with items were introduced before the given date.
+
+        :param date: The date to search the set.
+        :return: A copy of the OrderedSet with items added before the given date.
+        """
+        items = OrderedSet()
+        for item in [i for i in self.items if self[i] < date]:
+            items[item] = self.items[item]
+        return items
+
+    def until(self, date: datetime) -> 'OrderedSet':
+        """ Get a copy of the OrderedSet with items were introduced until the given date, including the same date.
+
+        :param date: The date to search the set.
+        :return: A copy of the OrderedSet with items added until the given date.
+        """
+        items = OrderedSet()
+        for item in [i for i in self.items if self[i] <= date]:
+            items[item] = self.items[item]
+        return items
+
+    def after(self, date: datetime) -> 'OrderedSet':
+        """ Get a copy of the OrderedSet with items were introduced after the given date.
+
+        :param date: The date to search the set.
+        :return: A copy of the OrderedSet with items added before the given date.
+        """
+        items = OrderedSet()
+        for item in [i for i in self.items if self[i] > date]:
+            items[item] = self.items[item]
+        return items
+
+    def since(self, date: datetime) -> 'OrderedSet':
+        """ Get a copy of the OrderedSet with items were introduced since the given date, including the same date.
+
+        :param date: The date to search the set.
+        :return: A copy of the OrderedSet with items added since the given date.
+        """
+        items = OrderedSet()
+        for item in [i for i in self.items if self[i] >= date]:
+            items[item] = self.items[item]
+        return items
 
     def remove(self, item: Hashable) -> None:
         """ Remove an element from the set.
@@ -344,6 +421,60 @@ class OrderedSet(collections.Set, Iterable):
         """
         return self.items.popitem(last)[0]
 
+    def remove_items(self, items: Iterable[Hashable], discard: bool = False) -> None:
+        """ Remove the given items from the set.
+
+        :param items: The items to remove.
+        :param discard: If True, do not raise a KeyError if the item is not found.
+        """
+        for item in items:
+            if discard:
+                self.discard(item)
+            else:
+                self.remove(item)
+
+    def remove_before(self, date: datetime, discard: bool = False) -> None:
+        """ Remove all the introduced items before the given date.
+
+        :param date: The date to search the set.
+        :param discard: If True, do not raise a KeyError if the item is not found.
+        """
+        self.remove_items([i for i in self.items if self[i] < date], discard)
+
+    def remove_until(self, date: datetime, discard: bool = False) -> None:
+        """ Remove all the introduced items  until the given date, including the same date.
+
+        :param date: The date to search the set.
+        :param discard: If True, do not raise a KeyError if the item is not found.
+        """
+        self.remove_items([i for i in self.items if self[i] <= date], discard)
+
+    def remove_after(self, date: datetime, discard: bool = False) -> None:
+        """ Remove all the introduced items after the given date.
+
+        :param date: The date to search the set.
+        :param discard: If True, do not raise a KeyError if the item is not found.
+        """
+        self.remove_items([i for i in self.items if self[i] > date], discard)
+
+    def remove_since(self, date: datetime, discard: bool = False) -> None:
+        """ Remove all the introduced items since the given date, including the same date.
+
+        :param date: The date to search the set.
+        :param discard: If True, do not raise a KeyError if the item is not found.
+        """
+        self.remove_items([i for i in self.items if self[i] >= date])
+
+    def first(self) -> 'Hashable':
+        """ Get the first element of the OrderedDict without removing it.
+
+        :return: The first element of the set.
+        """
+        if self.items:
+            return next(iter(self.items))
+        else:
+            raise KeyError('set is empty.')
+
     def discard(self, item: Hashable) -> None:
         """ Remove an element from the set.
 
@@ -357,9 +488,12 @@ class OrderedSet(collections.Set, Iterable):
 
     def copy(self) -> 'OrderedSet':
         """ Return a copy of the OrderedSet."""
-        return OrderedSet([k for k in self.items.keys()])
+        items = OrderedSet([k for k in self.items.keys()])
+        for item in items:
+            items[item] = self.items[item]
+        return items
 
-    def difference(self, other: collections.Set) -> 'OrderedSet':
+    def difference(self, other: set) -> 'OrderedSet':
         """ Get the elements in the set which are not present in the other set.
 
         :param other: The other set.
@@ -367,7 +501,7 @@ class OrderedSet(collections.Set, Iterable):
         """
         return OrderedSet(self.items.keys() - set(other))
 
-    def __sub__(self, other: collections.Set) -> 'OrderedSet':
+    def __sub__(self, other: set) -> 'OrderedSet':
         """ Get the elements in the set which are not present in the other set.
 
         :param other: The other set.
@@ -375,7 +509,7 @@ class OrderedSet(collections.Set, Iterable):
         """
         return self.difference(other)
 
-    def difference_update(self, other: collections.Set) -> None:
+    def difference_update(self, other: set) -> None:
         """ Remove the elements in the set which are not present in the other set.
 
         :param other: The other set.
@@ -384,7 +518,7 @@ class OrderedSet(collections.Set, Iterable):
         self.items = OrderedDict()
         self.update(items)
 
-    def intersection(self, other: collections.Set) -> 'OrderedSet':
+    def intersection(self, other: set) -> 'OrderedSet':
         """ Get the elements in the set which are present in the other set.
 
         :param other: The other set.
@@ -392,7 +526,7 @@ class OrderedSet(collections.Set, Iterable):
         """
         return OrderedSet(self.items.keys() & set(other))
 
-    def __and__(self, other: collections.Set) -> 'OrderedSet':
+    def __and__(self, other: set) -> 'OrderedSet':
         """ Get the elements in the set which are present in the other set.
 
         :param other: The other set.
@@ -400,7 +534,7 @@ class OrderedSet(collections.Set, Iterable):
         """
         return self.intersection(other)
 
-    def intersection_update(self, other: collections.Set) -> None:
+    def intersection_update(self, other: set) -> None:
         """ Remove the elements in the set which are not present in the other set.
 
         :param other: The other set.
@@ -409,7 +543,7 @@ class OrderedSet(collections.Set, Iterable):
         self.items = OrderedDict()
         self.update(items)
 
-    def union(self, other: collections.Set) -> 'OrderedSet':
+    def union(self, other: set) -> 'OrderedSet':
         """ Return the union of two sets as a new `OrderedSet`.
 
         :param other: The other set to union with.
@@ -417,7 +551,7 @@ class OrderedSet(collections.Set, Iterable):
         """
         return OrderedSet(self.items.keys() | set(other))
 
-    def __or__(self, other: collections.Set) -> 'OrderedSet':
+    def __or__(self, other: set) -> 'OrderedSet':
         """ Return the union of two sets as a new `OrderedSet`.
 
         :param other: The other set to union with.
@@ -425,7 +559,7 @@ class OrderedSet(collections.Set, Iterable):
         """
         return self.union(other)
 
-    def isdisjoint(self, other: collections.Set) -> bool:
+    def isdisjoint(self, other: set) -> bool:
         """ Check if the set is disjoint from the other set.
 
         :param other: The other set.
@@ -433,7 +567,7 @@ class OrderedSet(collections.Set, Iterable):
         """
         return not bool(self.items.keys() & set(other))
 
-    def issubset(self, other: collections.Set) -> bool:
+    def issubset(self, other: set) -> bool:
         """ Check if the set is a subset of the other set.
 
         :param other: The other set.
@@ -441,7 +575,7 @@ class OrderedSet(collections.Set, Iterable):
         """
         return self.items.keys() <= set(other)
 
-    def __le__(self, other: collections.Set) -> bool:
+    def __le__(self, other: set) -> bool:
         """ Check if the set is a subset of the other set.
 
         :param other: The other set.
@@ -449,7 +583,7 @@ class OrderedSet(collections.Set, Iterable):
         """
         return self.issubset(other)
 
-    def issuperset(self, other: collections.Set) -> bool:
+    def issuperset(self, other: set) -> bool:
         """ Check if the set is a superset of the other set.
 
         :param other: The other set.
@@ -457,7 +591,7 @@ class OrderedSet(collections.Set, Iterable):
         """
         return self.items.keys() >= set(other)
 
-    def __ge__(self, other: collections.Set) -> bool:
+    def __ge__(self, other: set) -> bool:
         """ Check if the set is a superset of the other set.
 
         :param other: The other set.
@@ -465,7 +599,7 @@ class OrderedSet(collections.Set, Iterable):
         """
         return self.issuperset(other)
 
-    def symmetric_difference(self, other: collections.Set) -> 'OrderedSet':
+    def symmetric_difference(self, other: set) -> 'OrderedSet':
         """ Return the symmetric difference of two sets as a new `OrderedSet`.
 
         :param other: The other set to take the symmetric difference with.
@@ -473,7 +607,7 @@ class OrderedSet(collections.Set, Iterable):
         """
         return OrderedSet(self.items.keys() ^ set(other))
 
-    def __pow__(self, other: collections.Set) -> 'OrderedSet':
+    def __pow__(self, other: set) -> 'OrderedSet':
         """ Return the symmetric difference of two sets as a new `OrderedSet`.
 
         :param other: The other set to take the symmetric difference with.
@@ -497,7 +631,7 @@ class OrderedSet(collections.Set, Iterable):
         """ Get the elements in the set. """
         return list(self.items.keys())
 
-    def __eq__(self, other: collections.Set) -> bool:
+    def __eq__(self, other: set) -> bool:
         """ Check if the set is equal to the other set.
 
         :param other: The other set.
@@ -505,7 +639,7 @@ class OrderedSet(collections.Set, Iterable):
         """
         return set(self.items.keys()) == set(other)
 
-    def __neq__(self, other: collections.Set) -> bool:
+    def __neq__(self, other: set) -> bool:
         """ Check if the set is not equal to the other set.
 
         :param other: The other set.
